<?php

namespace Drupal\content_moderation_reviewer\Form;

use Drupal\Core\Entity\ContentEntityForm;
use Drupal\Core\Entity\Entity\EntityFormDisplay;
use Drupal\Core\Form\FormStateInterface;

/**
 * The EntityModerationReviewerForm provides a simple UI for changing moderation state.
 *
 * This form is a copy of the content_moderation EntityModerationForm.
 *
 * @internal
 */
class EntityModerationReviewerForm extends ContentEntityForm {

  /**
   * {@inheritdoc}
   */
  public function getBaseFormId() {
    // We don't want the regular node form alter to fire.
    // Otherwise seven theme alters the form as well.
    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormDisplay(FormStateInterface $form_state) {
    // Create a transient display that is not persisted, but used only for
    // building the components required for the moderation form.
    $display = EntityFormDisplay::create([
      'targetEntityType' => $this->entity->getEntityTypeId(),
      'bundle' => $this->entity->bundle(),
    ]);
    $display->setComponent('moderation_state', [
      'type' => 'moderation_state_default',
      'weight' => 1,
      'settings' => [],
    ]);
    $display->setComponent('content_moderation_reviewer', [
      'weight' => 2,
      'type' => 'content_moderation_reviewer_autocomplete',
      'settings' => [
        'size' => 60,
        'match_operator' => 'CONTAINS',
        'placeholder' => ''
      ],
    ]);
    if ($log_message_field = $this->entity->getEntityType()->getRevisionMetadataKey('revision_log_message')) {
      $display->setComponent($log_message_field, [
        'weight' => 2,
        'type' => 'string_textarea',
        'settings' => [
          'rows' => 2,
        ],
      ]);
    }
    return $display;
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);
    // Hide the "advanced" part of the entity form so things like menu links
    // are not displayed.
    $form['advanced'] = [
      '#type' => 'container',
      '#access' => FALSE,
    ];
    // Use the entity_moderation_form template, which contains its own set of
    // classes.
    $form['#theme'] = ['content_moderation_reviewer_moderation_form'];
    $form['#attached']['library'][] = 'content_moderation/content_moderation';
    $form['#attributes']['class'] = [];

    unset($form['revision_log']['#group']);

    // Remove the field description from the "string_textfield" widget for the
    // revision log field (if one exists).
    unset($form[$this->entity->getEntityType()->getRevisionMetadataKey('revision_log_message')]['widget'][0]['value']['#description']);

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  protected function addRevisionableFormFields(array &$form) {
    parent::addRevisionableFormFields($form);
    // Show the revision fields as a simple container and ensure the revision
    // checkbox is not selectable.
    $form['revision_information']['#type'] = 'container';
    $form['revision']['#type'] = 'value';
    unset($form['revision_information']['#group']);
  }

  /**
   * {@inheritdoc}
   */
  protected function actions(array $form, FormStateInterface $form_state) {
    return [
      'submit' => [
        '#type' => 'submit',
        '#value' => $this->t('Apply'),
        '#submit' => ['::submitForm', '::save'],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
    $this->messenger()->addStatus($this->t('The moderation state has been updated.'));

    // The page we're on likely won't be visible if we just set the entity to
    // the default state, as we hide that latest-revision tab if there is no
    // pending revision. Redirect to the canonical URL instead, since that will
    // still exist.
    if ($this->entity->isDefaultRevision()) {
      $form_state->setRedirectUrl($this->entity->toUrl('canonical'));
    }
  }

}
