<?php

namespace Drupal\Tests\content_moderation_reviewer\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\node\Entity\Node;
use Drupal\node\Entity\NodeType;
use Drupal\node\NodeInterface;
use Drupal\user\Entity\Role;
use Drupal\user\Entity\User;
use Drupal\workflows\Entity\Workflow;

/**
 * @todo
 *
 * @see \Drupal\content_moderation\Plugin\Validation\Constraint\ModerationStateReviewerConstraintValidator
 *
 * @group content_moderation_reviewer
 */
class ConstraintValidatorTest extends CmrTestBase {

  public function testValidTransition() {
    $node = Node::create([
      'type' => 'page',
      'title' => 'Test title',
    ]);
    $node->get('moderation_state')->value = 'draft';
    $node->save();
    assert($node instanceof NodeInterface);

    // We want to move the moderation state to review.
    // Given that the only two users which should be allowed to be set
    // as reviewer are $role__review_published and $role__admin
    // as they are the only ones which can execute any further transition.

    $to_state_id = 'editorial_review';
    $data_provider = [
      'draft->editorial review user:test drafter 1' => [$this->users['test drafter 1'], FALSE],
      'draft->editorial review user:test drafter 2' => [$this->users['test drafter 2'], FALSE],
      'draft->editorial review user:test editorial_reviewer 1' => [$this->users['test editorial_reviewer 1'], TRUE],
      'draft->editorial review user:test editorial_reviewer 2' => [$this->users['test editorial_reviewer 2'], TRUE],
      'draft->editorial review user:test legal_reviewer 1' => [$this->users['test legal_reviewer 1'], FALSE],
      'draft->editorial review user:test legal_reviewer 2' => [$this->users['test legal_reviewer 2'], FALSE],
    ];

    $node->get('moderation_state')->value = $to_state_id;
    foreach ($data_provider as $key => list($user, $valid)) {
      $node->get('content_moderation_reviewer')->target_id = $user->id();

      if ($valid) {
        $this->assertEmpty($node->validate(), $key);
      }
      else {
        $this->assertNotEmpty($node->validate(), $key);
      }
    }

    $node->get('content_moderation_reviewer')->target_id = '';
    $node->get('moderation_state')->value = 'editorial_review';
    $node->save();
    $node = Node::load($node->id());

    $to_state_id = 'legal_review';
    $data_provider = [
      'editorial->legal_review review user:test drafter 1' => [$this->users['test drafter 1'], FALSE],
      'editorial->legal_review review user:test drafter 2' => [$this->users['test drafter 2'], FALSE],
      'editorial->legal_review review user:test editorial_reviewer 1' => [$this->users['test editorial_reviewer 1'], FALSE],
      'editorial->legal_review review user:test editorial_reviewer 2' => [$this->users['test editorial_reviewer 2'], FALSE],
      'editorial->legal_review review user:test legal_reviewer 1' => [$this->users['test legal_reviewer 1'], TRUE],
      'editorial->legal_review review user:test legal_reviewer 2' => [$this->users['test legal_reviewer 2'], TRUE],
    ];

    $node->get('moderation_state')->value = $to_state_id;
    foreach ($data_provider as $key => list($user, $valid)) {
      $node->get('content_moderation_reviewer')->target_id = $user->id();

      if ($valid) {
        $this->assertEmpty($node->validate(), $key);
      }
      else {
        $this->assertNotEmpty($node->validate(), $key);
      }
    }

    $node->get('content_moderation_reviewer')->target_id = '';
    $node->get('moderation_state')->value = 'legal_review';
    $node->save();
    $node = Node::load($node->id());

    $to_state_id = 'published';
    $data_provider = [
      'editorial->legal_review review user:test drafter 1' => [$this->users['test drafter 1'], FALSE],
      'editorial->legal_review review user:test drafter 2' => [$this->users['test drafter 2'], FALSE],
      'editorial->legal_review review user:test editorial_reviewer 1' => [$this->users['test editorial_reviewer 1'], FALSE],
      'editorial->legal_review review user:test editorial_reviewer 2' => [$this->users['test editorial_reviewer 2'], FALSE],
      'editorial->legal_review review user:test legal_reviewer 1' => [$this->users['test legal_reviewer 1'], FALSE],
      'editorial->legal_review review user:test legal_reviewer 2' => [$this->users['test legal_reviewer 2'], FALSE],
    ];

    $node->get('moderation_state')->value = $to_state_id;
    foreach ($data_provider as $key => list($user, $valid)) {
      $node->get('content_moderation_reviewer')->target_id = $user->id();

      if ($valid) {
        $this->assertEmpty($node->validate(), $key);
      }
      else {
        $this->assertNotEmpty($node->validate(), $key);
      }
    }
  }

}
